# Licensed to Elasticsearch B.V. under one or more contributor
# license agreements. See the NOTICE file distributed with
# this work for additional information regarding copyright
# ownership. Elasticsearch B.V. licenses this file to you under
# the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

require 'net/http'

endpoints = [
  'http://ruby_app:9292/',
  'http://ruby_app:9292/ingest',
  'http://ruby_app:9292/search/mario',
  'http://ruby_app:9292/search/sonic',
  'http://ruby_app:9292/search/donkeykong',
  'http://ruby_app:9292/search/bubsy',
  'http://ruby_app:9292/delete',
  'http://ruby_app:9292/update',
  'http://ruby_app:9292/error',
]

def sample_fibonacci
  [1, 2, 3, 5, 8, 13, 21].sample
end

while true
  sample_fibonacci.times do
    uri = URI(endpoints.sample)
    req = Net::HTTP::Get.new(uri)
    req['content-type'] = 'application/json'
    Thread.new do
      Net::HTTP.start(uri.hostname, uri.port) { |http| http.request(req) }
    end
    sleep 1
  end
end
