<%#
# Licensed to Elasticsearch B.V. under one or more contributor
# license agreements. See the NOTICE file distributed with
# this work for additional information regarding copyright
# ownership. Elasticsearch B.V. licenses this file to you under
# the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
%>
<%-
case @endpoint_name
when 'mtermvectors'
-%>
  if ids
    body = { :ids => ids }
  else
    body = arguments[:body]
  end
<%- when 'cluster.reroute', 'cluster.put_settings' %>
  body = arguments[:body] || {}
<%- when 'ml.find_file_structure' %>
  body = <%= __utils %>.__bulkify(arguments.delete(:body))
<%- else -%>
  <%= '  '*(@namespace_depth+3) %>body   = <%= @spec['body'].nil? ? 'nil' : 'arguments[:body]' %>
<%- end -%>
<%- if ['bulk', 'msearch', 'msearch_template', 'find_structure'].include? @method_name -%>
  <%= self.send("#{@method_name}_body_helper".to_s) %>
  headers = <%= __utils %>.ndjson_headers(headers)
  <%= '  '*(@namespace_depth+4) %>perform_request(method, path, params, payload, headers).body
<%- elsif @method_name == 'ping' -%>
  <%= ping_perform_request %>
<%- else -%>
  <%- if needs_ignore_404?(@endpoint_name) %>
    <%= __utils %>.__rescue_from_not_found do
      perform_request(method, path, params, body, headers).status == 200 ? true : false
    end
  <%- elsif needs_complex_ignore_404?(@endpoint_name) -%>
    if Array(arguments[:ignore]).include?(404)
      <%= __utils %>.__rescue_from_not_found { perform_request(method, path, params, body, headers).body }
    else
      perform_request(method, path, params, body, headers).body
    end
  <%- else -%>
      perform_request(method, path, params, body, headers).body
  <%- end -%>
<%- end -%>
<%= '  '*(@namespace_depth+3) %>end
