<%#
# Licensed to Elasticsearch B.V. under one or more contributor
# license agreements. See the NOTICE file distributed with
# this work for additional information regarding copyright
# ownership. Elasticsearch B.V. licenses this file to you under
# the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
%>

<%- unless @params.empty? -%>
  <%= '  '*(@namespace_depth+3) %># Register this action with its valid params when the module is loaded.
  <%= '  '*(@namespace_depth+3) %>#
  <%= '  '*(@namespace_depth+3) %># @since 6.2.0
  <%- if @endpoint_name == 'indices.stats' -%>
    <%= indices_stats_params_registry %>
  <%- else -%>
    <%= '  '*(@namespace_depth+3) %>ParamsRegistry.register(:<%= @method_name %>, [
    <%= '  '*(@namespace_depth+4) %><%= @params.keys.map { |k| ":#{k}" }.join(",\n#{'  '*(@namespace_depth+4)}") %>
    <%= '  '*(@namespace_depth+3) %>].freeze)
  <%- end -%>
<%- end -%>
